/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PreferencesFrame.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.preferences;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.proxy.AutoStartProxy;
import org.ow2.jonas.autostart.utility.SpringUtilities;

/**
 * This class creates main panel with ConfiguratorFrame fields.
 * @author skabore
 */
public class PreferencesFrame extends JDialog {

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The content panel.
     */
    private JPanel contentPanel;

    /**
     * The icon label.
     */
    private JLabel iconLabel;

    /**
     * The separator.
     */
    private JSeparator separator;

    /**
     * The text label.
     */
    private JLabel textLabel;

    /**
     * The title panel.
     */
    private JPanel titlePanel;

    /**
     * The applyProxSettings button.
     */
    private JButton jbApplyProxSettings;

    /**
     * The cancel button.
     */
    private JButton jbCancel;

    /**
     * The help button.
     */
    private JButton jbHelp;

    /**
     * The label.
     */
    private JLabel label;

    /**
     * The httpProxy textfield.
     */
    private JTextField httpProxy;

    /**
     * The httpProxy textfield.
     */
    private JTextField httpsProxy;

    /**
     * The ftpProxy textfield.
     */
    private JTextField ftpProxy;

    /**
     * The httpPort textfield.
     */
    private JFormattedTextField httpPort;

    /**
     * The httpsPort textfield.
     */
    private JFormattedTextField httpsPort;

    /**
     * The ftpPort textfield.
     */
    private JFormattedTextField ftpPort;

    /**
     * The user textfield.
     */
    private JTextField user;

    /**
     * The password textfield.
     */
    private JTextField password;

    /**
     * The tabbedPane.
     */
    private JTabbedPane tabbedPane;

    /**
     * The noProxy JRadioButton.
     */
    private JRadioButton noProxy;

    /**
     * The defineProxy JRadioButton.
     */
    private JRadioButton defineProxy;

    /**
     * The useForAllProtocols JCheckBox.
     */
    private JCheckBox useForAllProtocols;

    /**
     * The mainPanel.
     */
    private JPanel mainPanel;

    /**
     * The webContainerPanel.
     */
    private JPanel webContainerPanel;

    /**
     * The protocolPanel.
     */
    private JPanel protocolPanel;

    /**
     * The constructor.
     * @param builder the builder librairy.
     */
    public PreferencesFrame(final Builder builder) {
        super();
        setTitle("JOnAS AutoStart :: Preferences");
        this.builder = builder;
        contentPanel = getContentPanel();
        contentPanel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));

        ImageIcon icon = getImageIcon();

        titlePanel = new JPanel();
        textLabel = new JLabel();
        iconLabel = new JLabel();
        separator = new JSeparator();

        setLayout(new BorderLayout());

        titlePanel.setLayout(new java.awt.BorderLayout());
        titlePanel.setBackground(Color.gray);

        textLabel.setBackground(Color.gray);
        textLabel.setFont(new Font("MS Sans Serif", Font.BOLD, 14));
        textLabel.setText("Configure how \"Builder\" connects to the" + " Internet.");
        textLabel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
        textLabel.setOpaque(true);

        iconLabel.setBackground(Color.gray);
        if (icon != null) {
            iconLabel.setIcon(icon);
        }

        titlePanel.add(textLabel, BorderLayout.CENTER);
        titlePanel.add(separator, BorderLayout.SOUTH);

        add(titlePanel, BorderLayout.NORTH);
        JPanel secondaryPanel = new JPanel(new BorderLayout());
        secondaryPanel.add(contentPanel, BorderLayout.CENTER);
        add(secondaryPanel, BorderLayout.CENTER);
        pack();
        setVisible(true);
        setResizable(false);
        // setAlwaysOnTop(true);
        setEnabled(true);
        setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getPreferredSize().width) / 2, (Toolkit
                .getDefaultToolkit().getScreenSize().height - this.getPreferredSize().height) / 2);
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * @param builder the builder new value.
     */
    public void setBuilder(final Builder builder) {
        this.builder = builder;
    }

    /**
     * @return contentPanel1 the content panel.
     */
    private JPanel getContentPanel() {

        JPanel contentPanel1 = new JPanel(new BorderLayout());

        tabbedPane = new JTabbedPane();

        mainPanel = new JPanel(new SpringLayout());
        webContainerPanel = new JPanel(new BorderLayout());
        protocolPanel = new JPanel(new BorderLayout());

        httpProxy = new JTextField(20);
        httpsProxy = new JTextField(20);
        ftpProxy = new JTextField(20);
        httpPort = new JFormattedTextField();
        httpsPort = new JFormattedTextField();
        ftpPort = new JFormattedTextField();
        user = new JTextField(20);
        password = new JPasswordField(20);

        useForAllProtocols = new JCheckBox("Use this Proxy for all" + " protocols");
        noProxy = new JRadioButton("Direct connection to Internet");
        defineProxy = new JRadioButton("Manual proxy configuration");
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(noProxy);
        noProxy.setSelected(true);
        buttonGroup.add(defineProxy);

        jbApplyProxSettings = new JButton("Apply");
        jbCancel = new JButton("Cancel");
        jbHelp = new JButton("Help");
        jbApplyProxSettings.setMnemonic(KeyEvent.VK_A);
        jbCancel.setMnemonic(KeyEvent.VK_C);
        jbHelp.setMnemonic(KeyEvent.VK_H);

        addWebContainerComponents();
        addTabbedPane();

        contentPanel1.setLayout(new BorderLayout());
        contentPanel1.add(tabbedPane, BorderLayout.NORTH);

        initProxyValues();

        return contentPanel1;
    }

    /**
     * @return imageIcon. the image icon.
     */
    private ImageIcon getImageIcon() {
        return null;
    }

    /**
     * @return defineProxy
     */
    public JRadioButton getDefineProxy() {
        return defineProxy;
    }

    /**
     * @return ftpPort
     */
    public JFormattedTextField getFtpPort() {
        return ftpPort;
    }

    /**
     * @return ftpProxy
     */
    public JTextField getFtpProxy() {
        return ftpProxy;
    }

    /**
     * @return httpPort
     */
    public JFormattedTextField getHttpPort() {
        return httpPort;
    }

    /**
     * @return httpProxy
     */
    public JTextField getHttpProxy() {
        return httpProxy;
    }

    /**
     * @return httpsPort
     */
    public JFormattedTextField getHttpsPort() {
        return httpsPort;
    }

    /**
     * @return httpsProxy
     */
    public JTextField getHttpsProxy() {
        return httpsProxy;
    }

    /**
     * @return jbApplyProxSettings
     */
    public JButton getJbApplyProxSettings() {
        return jbApplyProxSettings;
    }

    /**
     * @return jbCancel
     */
    public JButton getJbCancel() {
        return jbCancel;
    }

    /**
     * @return jbHelp
     */
    public JButton getJbHelp() {
        return jbHelp;
    }

    /**
     * @return noProxy
     */
    public JRadioButton getNoProxy() {
        return noProxy;
    }

    /**
     * @return password
     */
    public JTextField getPassword() {
        return password;
    }

    /**
     * @return useForAllProtocols
     */
    public JCheckBox getUseForAllProtocols() {
        return useForAllProtocols;
    }

    /**
     * @return tabbedPane
     */
    public JTabbedPane getTabbedPane() {
        return tabbedPane;
    }

    /**
     * @return user
     */
    public JTextField getUser() {
        return user;
    }

    /**
     * add components to web container panel.
     */
    private void addWebContainerComponents() {
        JPanel networkPanel = new JPanel(new SpringLayout());
        networkPanel.add(noProxy);
        networkPanel.add(defineProxy);

        JPanel proxyPanel = new JPanel(new SpringLayout());
        proxyPanel.add(useForAllProtocols);

        JPanel proxyPanelComp = new JPanel(new SpringLayout());
        label = new JLabel("HTTP Proxy", JLabel.TRAILING);
        proxyPanelComp.add(label);
        label.setLabelFor(httpProxy);
        proxyPanelComp.add(httpProxy);
        label = new JLabel("Port");
        proxyPanelComp.add(label);
        label.setLabelFor(httpPort);
        httpPort.setPreferredSize(new Dimension(50, 10));
        proxyPanelComp.add(httpPort);

        label = new JLabel("SSL Proxy", JLabel.TRAILING);
        proxyPanelComp.add(label);
        label.setLabelFor(httpsProxy);
        proxyPanelComp.add(httpsProxy);
        label = new JLabel("Port");
        proxyPanelComp.add(label);
        label.setLabelFor(httpsPort);
        proxyPanelComp.add(httpsPort);

        label = new JLabel("FTP Proxy", JLabel.TRAILING);
        proxyPanelComp.add(label);
        label.setLabelFor(ftpProxy);
        proxyPanelComp.add(ftpProxy);
        label = new JLabel("Port");
        proxyPanelComp.add(label);
        label.setLabelFor(ftpPort);
        proxyPanelComp.add(ftpPort);

        label = new JLabel("User", JLabel.TRAILING);
        proxyPanelComp.add(label);
        label.setLabelFor(user);
        proxyPanelComp.add(user);
        proxyPanelComp.add(new JLabel(" "));
        proxyPanelComp.add(new JLabel(" "));

        label = new JLabel("Password", JLabel.TRAILING);
        proxyPanelComp.add(label);
        label.setLabelFor(password);
        proxyPanelComp.add(password);
        proxyPanelComp.add(new JLabel(" "));
        proxyPanelComp.add(new JLabel(" "));
        SpringUtilities.makeCompactGrid(proxyPanelComp, 5, 4, 6, 6, 6, 6);
        proxyPanel.add(proxyPanelComp);
        SpringUtilities.makeCompactGrid(proxyPanel, 2, 1, 6, 6, 6, 6);
        networkPanel.add(proxyPanel);

        JPanel buttons = new JPanel();
        buttons.add(jbApplyProxSettings);
        buttons.add(jbCancel);
        // buttons.add(jbHelp);
        networkPanel.add(buttons);
        SpringUtilities.makeCompactGrid(networkPanel, 4, 1, 6, 6, 6, 6);
        webContainerPanel.add(networkPanel, BorderLayout.NORTH);
        showProxyFields(false);
    }

    /**
     * add tabbedPane to the frame.
     */
    private void addTabbedPane() {
        tabbedPane.addTab("Network", webContainerPanel);
        tabbedPane.setMnemonicAt(0, KeyEvent.VK_N);
        tabbedPane.addTab("...", protocolPanel);
        tabbedPane.setMnemonicAt(1, KeyEvent.VK_C);
        mainPanel.add(tabbedPane);
    }

    /**
     * Reset the proxy fields.
     */
    public void resetProxyFields() {
        user.setText("");
        password.setText("");
        useForAllProtocols.setSelected(true);
        httpPort.setText("0");
        httpsPort.setText("0");
        httpProxy.setText("");
        httpsProxy.setText("");
        ftpPort.setText("0");
        ftpProxy.setText("");
        noProxy.setSelected(true);
        defineProxy.setSelected(false);
        showProxyFields(false);
    }

    /**
     * Show or hide the proxy fields.
     * @param enabled if true, prxy fields are showed, hided otherwise
     */
    public void showProxyFields(final boolean enabled) {
        user.setEnabled(enabled);
        password.setEnabled(enabled);
        useForAllProtocols.setEnabled(enabled);
        httpPort.setEnabled(enabled);
        httpsPort.setEnabled(enabled);
        httpProxy.setEnabled(enabled);
        httpsProxy.setEnabled(enabled);
        ftpPort.setEnabled(enabled);
        ftpProxy.setEnabled(enabled);
    }

    /**
     * Show or hide the proxy fields.
     * @param enabled if true, prxy fields are showed, hided otherwise
     */
    public void showOtherProtocolsFields(final boolean enabled) {
        httpsPort.setEnabled(enabled);
        httpsProxy.setEnabled(enabled);
        ftpPort.setEnabled(enabled);
        ftpProxy.setEnabled(enabled);
    }

    /**
     * Add actionListener to the buttons.
     * @param listener the listener.
     */
    public void addActionListener(final ActionListener listener) {
        noProxy.addActionListener(listener);
        defineProxy.addActionListener(listener);
        useForAllProtocols.addActionListener(listener);
        jbApplyProxSettings.addActionListener(listener);
        jbCancel.addActionListener(listener);
        jbHelp.addActionListener(listener);
    }

    /**
     * Add keyListener to the buttons.
     * @param listener the listener.
     */
    @Override
    public void addKeyListener(final KeyListener listener) {
        httpPort.addKeyListener(listener);
        httpsPort.addKeyListener(listener);
        ftpPort.addKeyListener(listener);
    }

    /**
     * Initialize the proxy value.
     */
    public void initProxyValues() {
        AutoStartProxy proxy = builder.getAutoStartProxy();
        httpProxy.setText(proxy.address);
        httpPort.setText(proxy.port);
        user.setText(proxy.user);
        password.setText(proxy.password);
    }

    /**
     * Copy HTTP address to set hTTPS, FTP address field.
     */
    void setOthersFieldsValue() {
        httpsProxy.setText(httpProxy.getText());
        httpsPort.setText(httpPort.getText());
        ftpProxy.setText(httpProxy.getText());
        ftpPort.setText(httpPort.getText());
    }

    /**
     * Clean HTTPS, FTP address field.
     */
    void cleanOthersFieldsValue() {
        httpsProxy.setText("");
        httpsPort.setText("0");
        ftpProxy.setText("");
        ftpPort.setText("0");
    }
}
