/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PreferencesFrameDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.preferences;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JOptionPane;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.proxy.AutoStartProxy;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instanciates PreferencesFrame.
 * @author skabore
 */
public class PreferencesFrameDescriptor implements ActionListener, KeyListener {

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The PreferencesFrame.
     */
    private PreferencesFrame preferencesFrame;

    /**
     * The constructeur.
     * @param builder the builder librairy
     */
    public PreferencesFrameDescriptor(final Builder builder) {
        this.builder = builder;
        preferencesFrame = new PreferencesFrame(builder);
        preferencesFrame.addActionListener(this);
        preferencesFrame.addKeyListener(this);
        // preferencesFrame.addListSelectionListener(this);
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * Set the builder.
     * @param builder the new builder
     */
    public void setBuilder(final Builder builder) {
        this.builder = builder;
    }

    /**
     * @return preferencesFrame
     */
    public PreferencesFrame getPreferencesFrame() {
        return preferencesFrame;
    }

    /**
     * Set the preferencesFrame.
     * @param preferencesFrame the preferencesFrame new value
     */
    public void setPreferencesFrame(final PreferencesFrame preferencesFrame) {
        this.preferencesFrame = preferencesFrame;
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == preferencesFrame.getNoProxy()) {
            preferencesFrame.showProxyFields(false);
        } else if (event.getSource() == preferencesFrame.getDefineProxy()) {
            preferencesFrame.showProxyFields(true);
        } else if (event.getSource() == preferencesFrame.getUseForAllProtocols()) {
            preferencesFrame.showOtherProtocolsFields(!preferencesFrame.getUseForAllProtocols().isSelected());
            if (preferencesFrame.getUseForAllProtocols().isSelected()) {
                preferencesFrame.setOthersFieldsValue();
            } else {
                preferencesFrame.cleanOthersFieldsValue();
            }
        } else if (event.getSource() == preferencesFrame.getJbApplyProxSettings()) {
            saveProxSettings();
            preferencesFrame.resetProxyFields();
            preferencesFrame.dispose();
        } else if (event.getSource() == preferencesFrame.getJbCancel()) {
            preferencesFrame.resetProxyFields();
            preferencesFrame.dispose();
        }
    }

    /**
     * Save the proxy settings.
     */
    private void saveProxSettings() {
        AutoStartProxy proxy = new AutoStartProxy();
        if (preferencesFrame.getDefineProxy().isSelected()) {
            proxy.address = preferencesFrame.getHttpProxy().getText();
            proxy.port = preferencesFrame.getHttpPort().getText();
            proxy.user = preferencesFrame.getUser().getText();
            proxy.password = preferencesFrame.getPassword().getText();
        }
        Utility.initProxyValues(proxy);
        Utility.persitProxyValues(proxy);
        builder.setAutoStartProxy(proxy);
        preferencesFrame.setBuilder(builder);
    }

    /**
     * Show a message dialog.
     * @param message the message to print.
     */
    private void showMessage(final String message) {
        JOptionPane.showMessageDialog(preferencesFrame, message, "Informations", JOptionPane.INFORMATION_MESSAGE);
    }

    public void keyTyped(KeyEvent event) {
    }

    public void keyPressed(KeyEvent event) {
    }

    public void keyReleased(KeyEvent event) {
        if (event.getSource() == preferencesFrame.getHttpPort()) {
            String number = preferencesFrame.getHttpPort().getText();
            number = Utility.checkIntegerNumberFormat(number);
            preferencesFrame.getHttpPort().setText(number);
        } else if (event.getSource() == preferencesFrame.getHttpsPort()) {
            String number = preferencesFrame.getHttpsPort().getText();
            number = Utility.checkIntegerNumberFormat(number);
            preferencesFrame.getHttpsPort().setText(number);
        } else if (event.getSource() == preferencesFrame.getFtpPort()) {
            String number = preferencesFrame.getFtpPort().getText();
            number = Utility.checkIntegerNumberFormat(number);
            preferencesFrame.getFtpPort().setText(number);
        }
    }
}
